
/* ************************************************************************* */
/*
 *	test_uuid_hash.c
 *
 *	(C) 2010 VOCAL Technologies, Ltd.
 *
 *	This software is intended for use as a reference/standard implementation,
 *	and has been placed into the public domain by the copyright holder:
 *
 *	VOCAL Technologies, Ltd.
 *	90A John Muir Drive.
 *	Buffalo, NY  14228
 *
 *	Maintainer:  Dennis Towne
 */
/* ************************************************************************* */

#include <stdio.h>

#include "system.h"
#include "uuid_hash.h"
#include "util_tests.h"


/******************************************************************************/
/* uuid display/generate helpers */
/******************************************************************************/

void generate_and_print_uuid(struct test_data *td) {
	char uuid[UUID_HASH_STRING_LENGTH + 1];
	char temp;

	generate_test_uuid(td, uuid);

	printf("Patient UUID - %s\n", uuid);

	temp = uuid[0];
	if ((temp >= '0' && temp <= '9') || (temp >= 'a' && temp <= 'f')) {
		if (td->patient_ssn[0]) {
			printf("Error in generate_and_print_uuid(), uuid_create() generated bad leading digit for patient SSN present.\n");
		}
	}
	else {
		if (!td->patient_ssn[0]) {
			printf("Error in generate_and_print_uuid(), uuid_create() generated bad leading digit for patient SSN missing.\n");
		}
	}
}

void print_uuid_parms(struct test_data *td) {
	printf("Patient Name        - %s\n", td->patient_name);
	if (td->patient_ssn[0]) {
		printf("Patient SSN         - %s\n", td->patient_ssn);
	}
	printf("Patient gender      - %d\n", td->patient_gender);
	printf("Patient birth year  - %d\n", td->patient_year);
	printf("Patient birth month - %d\n", td->patient_month);
	printf("Patient birth day   - %d\n", td->patient_day);

	if (td->policy_ssn[0]) {
		printf("Policy Name         - %s\n", td->policy_name);
		printf("Policy SSN          - %s\n", td->policy_ssn);
		printf("Policy gender       - %d\n", td->policy_gender);
		printf("Policy birth year   - %d\n", td->policy_year);
		printf("Policy birth month  - %d\n", td->policy_month);
		printf("Policy birth day    - %d\n", td->policy_day);
	}
}


/******************************************************************************/
/* demo routine - patient SSN available */
/******************************************************************************/

static void demonstrate_ssn_uuid_generation() {
	struct test_data td;
	init_test_data(&td);

	/*********************************************************************/
	/* demonstrate changes in name */
	/*********************************************************************/

	strxcpy(td.patient_name, "dennis", sizeof(td.patient_name));
	sprintf(td.patient_ssn, "%09d", 123456789);
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1970;
	td.patient_month  = UUID_MONTH_OCTOBER;
	td.patient_day    = 17;

	printf("\n");
	printf("SSN Present - Section 1 - demonstration of name differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "michael", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "micheal", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "nicolas", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "amy", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in ssn */
	/*********************************************************************/

	strxcpy(td.patient_name, "dennis", sizeof(td.patient_name));
	sprintf(td.patient_ssn, "%09d", 123456789);
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1970;
	td.patient_month  = UUID_MONTH_OCTOBER;
	td.patient_day    = 17;

	printf("\n");
	printf("SSN Present - Section 2 - demonstration of SSN differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("SSN          - %s   :  ", td.patient_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.patient_ssn, "%09d", 123456788);
	printf("SSN          - %s   :  ", td.patient_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.patient_ssn, "%09d", 123456787);
	printf("SSN          - %s   :  ", td.patient_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.patient_ssn, "%09d", 123416789);
	printf("SSN          - %s   :  ", td.patient_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.patient_ssn, "%09d", 223456789);
	printf("SSN          - %s   :  ", td.patient_ssn);
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in gender */
	/*********************************************************************/

	strxcpy(td.patient_name, "dennis", sizeof(td.patient_name));
	sprintf(td.patient_ssn, "%09d", 123456789);
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1970;
	td.patient_month  = UUID_MONTH_OCTOBER;
	td.patient_day    = 17;

	printf("\n");
	printf("SSN Present - Section 3 - demonstration of gender differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("Gender       - MALE        :  ");
	generate_and_print_uuid(&td);

	td.patient_gender = UUID_GENDER_FEMALE;
	printf("Gender       - FEMALE      :  ");
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in birth year */
	/*********************************************************************/

	strxcpy(td.patient_name, "dennis", sizeof(td.patient_name));
	sprintf(td.patient_ssn, "%09d", 123456789);
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1970;
	td.patient_month  = UUID_MONTH_OCTOBER;
	td.patient_day    = 17;

	printf("\n");
	printf("SSN Present - Section 4 - demonstration of birth year differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.patient_year <= 1980) {
		printf("Birth Year   - %-10d  :  ", td.patient_year);
		generate_and_print_uuid(&td);
		td.patient_year++;
	}


	/*********************************************************************/
	/* demonstrate changes in birth month */
	/*********************************************************************/

	strxcpy(td.patient_name, "dennis", sizeof(td.patient_name));
	sprintf(td.patient_ssn, "%09d", 123456789);
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1970;
	td.patient_month  = UUID_MONTH_JANUARY;
	td.patient_day    = 17;

	printf("\n");
	printf("SSN Present - Section 5 - demonstration of birth month differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.patient_month <= UUID_MONTH_DECEMBER) {
		printf("Birth Month  - %-10d  :  ", td.patient_month);
		generate_and_print_uuid(&td);
		td.patient_month++;
	}


	/*********************************************************************/
	/* demonstrate changes in birth day */
	/*********************************************************************/

	strxcpy(td.patient_name, "dennis", sizeof(td.patient_name));
	sprintf(td.patient_ssn, "%09d", 123456789);
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1970;
	td.patient_month  = UUID_MONTH_OCTOBER;
	td.patient_day    = 17;

	printf("\n");
	printf("SSN Present - Section 6 - demonstration of birth day differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.patient_day < 30) {
		printf("Birth Day    - %-10d  :  ", td.patient_day);
		generate_and_print_uuid(&td);
		td.patient_day++;
	}
}


/******************************************************************************/
/* demo routines - patient SSN NOT available */
/******************************************************************************/

static void demonstrate_nossn_uuid_generation_1() {
	struct test_data td;
	init_test_data(&td);

	/* set up base data for the patient */
	strxcpy(td.patient_name, "gary", sizeof(td.patient_name));
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1990;
	td.patient_month  = UUID_MONTH_AUGUST;
	td.patient_day    = 3;


	/*********************************************************************/
	/* demonstrate changes in name */
	/*********************************************************************/

	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_OCTOBER;
	td.policy_day    = 17;

	printf("\n");
	printf("NO SSN PRESENT - Section 1 - demonstration of policy holder name differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("Name         - %-10s  :  ", td.policy_name);
	generate_and_print_uuid(&td);

	strxcpy(td.policy_name, "michael", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.policy_name);
	generate_and_print_uuid(&td);

	strxcpy(td.policy_name, "micheal", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.policy_name);
	generate_and_print_uuid(&td);

	strxcpy(td.policy_name, "nicolas", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.policy_name);
	generate_and_print_uuid(&td);

	strxcpy(td.policy_name, "amy", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.policy_name);
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in ssn */
	/*********************************************************************/

	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_OCTOBER;
	td.policy_day    = 17;

	printf("\n");
	printf("NO SSN PRESENT - Section 2 - demonstration of policy holder SSN differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("SSN          - %s   :  ", td.policy_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.policy_ssn , "%09d",123456788);
	printf("SSN          - %s   :  ", td.policy_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.policy_ssn , "%09d",123456787);
	printf("SSN          - %s   :  ", td.policy_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.policy_ssn , "%09d",123416789);
	printf("SSN          - %s   :  ", td.policy_ssn);
	generate_and_print_uuid(&td);

	sprintf(td.policy_ssn , "%09d",223456789);
	printf("SSN          - %s   :  ", td.policy_ssn);
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in gender */
	/*********************************************************************/

	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_OCTOBER;
	td.policy_day    = 17;

	printf("\n");
	printf("NO SSN PRESENT - Section 3 - demonstration of policy holder gender differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("Gender       - MALE        :  ");
	generate_and_print_uuid(&td);

	td.policy_gender = UUID_GENDER_FEMALE;
	printf("Gender       - FEMALE      :  ");
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in birth year */
	/*********************************************************************/

	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_OCTOBER;
	td.policy_day    = 17;

	printf("\n");
	printf("NO SSN PRESENT - Section 4 - demonstration of policy holder birth year differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.policy_year <= 1980) {
		printf("Birth Year   - %-10d  :  ", td.policy_year);
		generate_and_print_uuid(&td);
		td.policy_year++;
	}


	/*********************************************************************/
	/* demonstrate changes in birth month */
	/*********************************************************************/

	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_JANUARY;
	td.policy_day    = 17;

	printf("\n");
	printf("NO SSN PRESENT - Section 5 - demonstration of policy holder birth month differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.policy_month <= UUID_MONTH_DECEMBER) {
		printf("Birth Month  - %-10d  :  ", td.policy_month);
		generate_and_print_uuid(&td);
		td.policy_month++;
	}


	/*********************************************************************/
	/* demonstrate changes in birth day */
	/*********************************************************************/

	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_OCTOBER;
	td.policy_day    = 17;

	printf("\n");
	printf("NO SSN PRESENT - Section 6 - demonstration of policy holder birth day differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.policy_day < 30) {
		printf("Birth Day    - %-10d  :  ", td.policy_day);
		generate_and_print_uuid(&td);
		td.policy_day++;
	}
}

static void demonstrate_nossn_uuid_generation_2() {
	struct test_data td;
	init_test_data(&td);

	/* set up base data for the policy holder */
	strxcpy(td.policy_name, "dennis", sizeof(td.patient_name));
	sprintf(td.policy_ssn , "%09d",123456789);
	td.policy_gender = UUID_GENDER_MALE;
	td.policy_year   = 1970;
	td.policy_month  = UUID_MONTH_OCTOBER;
	td.policy_day    = 17;


	/*********************************************************************/
	/* demonstrate changes in name */
	/*********************************************************************/

	strxcpy(td.patient_name, "gary", sizeof(td.patient_name));
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1990;
	td.patient_month  = UUID_MONTH_AUGUST;
	td.patient_day    = 3;

	printf("\n");
	printf("NO SSN PRESENT - Section 1 - demonstration of patient name differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "michael", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "micheal", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "nicolas", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);

	strxcpy(td.patient_name, "amy", sizeof(td.patient_name));
	printf("Name         - %-10s  :  ", td.patient_name);
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in gender */
	/*********************************************************************/

	strxcpy(td.patient_name, "gary", sizeof(td.patient_name));
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1990;
	td.patient_month  = UUID_MONTH_AUGUST;
	td.patient_day    = 3;

	printf("\n");
	printf("NO SSN PRESENT - Section 3 - demonstration of patient gender differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	printf("Gender       - MALE        :  ");
	generate_and_print_uuid(&td);

	td.patient_gender = UUID_GENDER_FEMALE;
	printf("Gender       - FEMALE      :  ");
	generate_and_print_uuid(&td);


	/*********************************************************************/
	/* demonstrate changes in birth year */
	/*********************************************************************/

	strxcpy(td.patient_name, "gary", sizeof(td.patient_name));
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1990;
	td.patient_month  = UUID_MONTH_AUGUST;
	td.patient_day    = 3;

	printf("\n");
	printf("NO SSN PRESENT - Section 4 - demonstration of patient birth year differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.patient_year <= 2010) {
		printf("Birth Year   - %-10d  :  ", td.patient_year);
		generate_and_print_uuid(&td);
		td.patient_year++;
	}


	/*********************************************************************/
	/* demonstrate changes in birth month */
	/*********************************************************************/

	strxcpy(td.patient_name, "gary", sizeof(td.patient_name));
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1990;
	td.patient_month  = UUID_MONTH_AUGUST;
	td.patient_day    = 3;

	printf("\n");
	printf("NO SSN PRESENT - Section 5 - demonstration of patient birth month differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.patient_month <= UUID_MONTH_DECEMBER) {
		printf("Birth Month  - %-10d  :  ", td.patient_month);
		generate_and_print_uuid(&td);
		td.patient_month++;
	}


	/*********************************************************************/
	/* demonstrate changes in birth day */
	/*********************************************************************/

	strxcpy(td.patient_name, "gary", sizeof(td.patient_name));
	td.patient_gender = UUID_GENDER_MALE;
	td.patient_year   = 1990;
	td.patient_month  = UUID_MONTH_AUGUST;
	td.patient_day    = 3;

	printf("\n");
	printf("NO SSN PRESENT - Section 6 - demonstration of patient birth day differences\n");
	printf("-----------------------------------------------------------\n");
	print_uuid_parms(&td);
	printf("\n");

	while(td.patient_day < 30) {
		printf("Birth Day    - %-10d  :  ", td.patient_day);
		generate_and_print_uuid(&td);
		td.patient_day++;
	}
}


/******************************************************************************/
/* main routine and usage */
/******************************************************************************/

int main(int argc, char *argv[]) {
	printf("This program demonstrates the interface to the reference patient\n");
	printf("UUID library, and shows created UUIDs for a handful of cases.\n");

	demonstrate_ssn_uuid_generation();
	demonstrate_nossn_uuid_generation_1();
	demonstrate_nossn_uuid_generation_2();

	return 0;
}

