
/* ************************************************************************* */
/*
 *	util_tests.c
 *
 *	(C) 2010 VOCAL Technologies, Ltd.
 *
 *	This software is intended for use as a reference/standard implementation,
 *	and has been placed into the public domain by the copyright holder:
 *
 *	VOCAL Technologies, Ltd.
 *	90A John Muir Drive.
 *	Buffalo, NY  14228
 *
 *	Maintainer:  Dennis Towne
 */
/* ************************************************************************* */


#include <string.h>

#include "system.h"
#include "util_tests.h"
#include "uuid_hash.h"


/******************************************************************************/
/* various utilities for handling test data structures */
/******************************************************************************/

void init_test_data(struct test_data *td) {
	td->record_number = 0;

	td->policy_name[0] = 0;
	td->policy_ssn[0] = 0;
	td->policy_gender = 0;
	td->policy_year = 0;
	td->policy_month = 0;
	td->policy_day = 0;

	td->patient_name[0] = 0;
	td->patient_ssn[0] = 0;
	td->patient_gender = 0;
	td->patient_year = 0;
	td->patient_month = 0;
	td->patient_day = 0;

	td->uuid[0] = 0;
}

int generate_test_uuid(struct test_data *td, char *uuid_dest) {
	struct uuid_hash_data ud;

	uuid_initialize(&ud);

	if (td->policy_name && td->policy_name[0] != 0) {
		uuid_hash_set_policy_name         (&ud, td->policy_name);
	}
	if (td->policy_ssn[0]) {
		uuid_hash_set_policy_ssn          (&ud, td->policy_ssn);
	}
	if (td->policy_gender) {
		uuid_hash_set_policy_gender       (&ud, td->policy_gender);
	}
	if (td->policy_year) {
		uuid_hash_set_policy_birth_year   (&ud, td->policy_year);
	}
	if (td->policy_month) {
		uuid_hash_set_policy_birth_month  (&ud, td->policy_month);
	}
	if (td->policy_day) {
		uuid_hash_set_policy_birth_day    (&ud, td->policy_day);
	}

	if (td->patient_name && td->patient_name[0] != 0) {
		uuid_hash_set_patient_name        (&ud, td->patient_name);
	}
	if (td->patient_ssn[0]) {
		uuid_hash_set_patient_ssn         (&ud, td->patient_ssn);
	}
	if (td->patient_gender) {
		uuid_hash_set_patient_gender      (&ud, td->patient_gender);
	}
	if (td->patient_year) {
		uuid_hash_set_patient_birth_year  (&ud, td->patient_year);
	}
	if (td->patient_month) {
		uuid_hash_set_patient_birth_month (&ud, td->patient_month);
	}
	if (td->patient_day) {
		uuid_hash_set_patient_birth_day   (&ud, td->patient_day);
	}

	if (!uuid_create(&ud)) {
		uuid_dest[0] = 0;
		printf("Error in generate_test_uuid(), uuid_create() failed!\n");
		return 0;
	}

	strcpy(uuid_dest, ud.uuid);
	return 1;
}


/******************************************************************************/
/* file load utilities */
/******************************************************************************/

int fget_line(FILE *stream, char *dest, int dest_size) {
	if(fgets(dest, dest_size, stream) == NULL) {
		return 0;
	}

	/* strip trailing \n if necessary */
	while(*dest != 0 && *dest != '\n' && *dest != '\r') { 
		dest++;
	}
	*dest = 0;

	return 1;
}


/******************************************************************************/
/* various string handling utilities */
/******************************************************************************/

int wordcmp(const char *data, const char *ref) {
	while(*data == *ref && *ref != 0) {
		data++;
		ref++;
	}

	if (*data != 0 && *data != ' ' && *data != '\n' && *data != '\r' && *data != '\t') {
		/* if there's still remaining non-whitespace in the data word, bail */
		return 0;
	}
	if (*ref == 0) {
		/* if data is whitespace and ref is at its end, success */
		return 1;
	}
	return 0;
}

char *next_arg(char *string) {
	if (string == NULL) {
		return NULL;
	}

	/* nuke spaces at the start of the string */
	while (*string == ' ' || *string == '\n' || *string == '\t') {
		string++;
	}
	if (*string == 0) {
		return NULL;
	}

	/* skip past all until you hit a space or termination */
	while (*string != ' ' && *string != '\n' && *string != '\t' && *string != 0) {
		string++;
	}

	/* nuke spaces at the end of first string */
	while (*string == ' ' || *string == '\n' || *string == '\t') {
		string++;
	}
	if (*string == 0) {
		return NULL;
	}

	/* return the next argument */
	return string;
}

void strxcpy(char *dest, const char *src, unsigned int dest_len) {
	/* I really don't know why strncpy doesn't have these semantics */
	strncpy(dest, src, dest_len);
	if (dest_len > 0) {
		dest[dest_len - 1] = 0;
	}
}
