
/* ************************************************************************* */
/*
 *	uuid_hash.h
 *
 *	(C) 2010 VOCAL Technologies, Ltd.
 *
 *	This software is intended for use as a reference/standard implementation,
 *	and has been placed into the public domain by the copyright holder:
 *
 *	VOCAL Technologies, Ltd.
 *	90A John Muir Drive.
 *	Buffalo, NY  14228
 *
 *	Maintainer:  Dennis Towne
 */
/* ************************************************************************* */


#define UUID_FIRST_NAME_LEN      20
#define UUID_SSN_LEN              9

#define UUID_GENDER_MALE          1
#define UUID_GENDER_FEMALE        2

#define UUID_HASH_STRING_LENGTH  12

#define UUID_MONTH_JANUARY        1
#define UUID_MONTH_FEBUARY        2
#define UUID_MONTH_MARCH          3
#define UUID_MONTH_APRIL          4
#define UUID_MONTH_MAY            5
#define UUID_MONTH_JUNE           6
#define UUID_MONTH_JULY           7
#define UUID_MONTH_AUGUST         8
#define UUID_MONTH_SEPTEMBER      9
#define UUID_MONTH_OCTOBER       10
#define UUID_MONTH_NOVEMBER      11
#define UUID_MONTH_DECEMBER      12

#define UUID_FLAG_PATIENT_SSN        0x0001
#define UUID_FLAG_PATIENT_FN         0x0002
#define UUID_FLAG_PATIENT_DOB        0x0004
#define UUID_FLAG_PATIENT_GENDER     0x0008
#define UUID_FLAG_PATIENT_YOB        0x0010
#define UUID_FLAG_PATIENT_MOB        0x0020
#define UUID_PATIENT_FLAGS_OK        0x003f		/* require all patient fields */

#define UUID_FLAG_POLICY_SSN         0x0100
#define UUID_FLAG_POLICY_FN          0x0200
#define UUID_FLAG_POLICY_DOB         0x0400
#define UUID_FLAG_POLICY_GENDER      0x0800
#define UUID_FLAG_POLICY_YOB         0x1000
#define UUID_FLAG_POLICY_MOB         0x2000
#define UUID_PATIENT_NOSSN_FLAGS_OK  0x3f3e		/* require all policy holder fields, and all patient fields except SSN */

#define UUID_PATIENT_SSN_FLAGS       0x0101		/* ssn fields - for detecting bogus/all 0 uuid generation */

#define UUID_H1_INPUT_LEN          (UUID_FIRST_NAME_LEN+UUID_SSN_LEN+1)
#define UUID_H1_OUTPUT_NIBBLES    9		/* 36 bits */

#define UUID_H2_INPUT_LEN          (UUID_H1_OUTPUT_NIBBLES+1+1+2)	/* GGMMYYYY */

#define UUID_H3_BLOCK_1_LEN        (UUID_FIRST_NAME_LEN+UUID_SSN_LEN+1+1+1+2)	/* GGDDMMYYYY */
#define UUID_H3_BLOCK_2_LEN        (UUID_FIRST_NAME_LEN+1+1+1+2)				/* GGDDMMYYYY */
#define UUID_H3_MAX_INPUT_LEN      (UUID_H3_BLOCK_1_LEN)

struct uuid_hash_data {
	int flags;

	/* policy holder - private fields */
	char policy_first_name[UUID_FIRST_NAME_LEN];   /* string data */
	char policy_ssn[UUID_SSN_LEN];                 /* string data */
	int  policy_day_of_birth;                       /* 1-31 */

	/* policy holder - public fields */
	int policy_gender;                             /* 0-1 */
	int policy_year_of_birth;                      /* 0-9999 */
	int policy_month_of_birth;                     /* 1-12 */

	/* patient - private fields */
	char patient_first_name[UUID_FIRST_NAME_LEN];   /* string data */
	char patient_ssn[UUID_SSN_LEN];                 /* string data */
	int  patient_day_of_birth;                       /* 1-31 */

	/* patient - public fields */
	int patient_gender;                             /* 0-1 */
	int patient_year_of_birth;                      /* 0-9999 */
	int patient_month_of_birth;                     /* 1-12 */

	/* hash output */
	char uuid[UUID_HASH_STRING_LENGTH + 1];            /* string data */
};

LINKSPEC void uuid_initialize(struct uuid_hash_data *ud);

LINKSPEC int uuid_hash_set_patient_name        (struct uuid_hash_data *ud, const char *name);
LINKSPEC int uuid_hash_set_patient_ssn         (struct uuid_hash_data *ud, const char *nns);
LINKSPEC int uuid_hash_set_patient_gender      (struct uuid_hash_data *ud, int gender);
LINKSPEC int uuid_hash_set_patient_birth_year  (struct uuid_hash_data *ud, int yob);
LINKSPEC int uuid_hash_set_patient_birth_month (struct uuid_hash_data *ud, int mob);
LINKSPEC int uuid_hash_set_patient_birth_day   (struct uuid_hash_data *ud, int dob);

LINKSPEC int uuid_hash_set_policy_name        (struct uuid_hash_data *ud, const char *name);
LINKSPEC int uuid_hash_set_policy_ssn         (struct uuid_hash_data *ud, const char *nns);
LINKSPEC int uuid_hash_set_policy_gender      (struct uuid_hash_data *ud, int gender);
LINKSPEC int uuid_hash_set_policy_birth_year  (struct uuid_hash_data *ud, int yob);
LINKSPEC int uuid_hash_set_policy_birth_month (struct uuid_hash_data *ud, int mob);
LINKSPEC int uuid_hash_set_policy_birth_day   (struct uuid_hash_data *ud, int dob);

LINKSPEC int uuid_create(struct uuid_hash_data *ud);

